/**************************************************************************/
/*                                                                        */
/* mpcx_short                                                             */
/*                                                                        */
/* computes products and inverses modulo powers of X                      */
/*                                                                        */
/* Copyright (C) 2007 Andreas Enge                                        */
/*                                                                        */
/* Last changes: September 17, 2007                                       */
/*                                                                        */
/**************************************************************************/

#include "mpcx-impl.h"

void mpcx_smul (mpcx_ptr h, mpcx_ptr f, mpcx_ptr g, const int exp) {
   /* computes the short product h = f*g mod X^exp; for the time being,   */
   /* by a complete product and a truncation                              */

   int f_deg, g_deg, h_deg;

   f_deg = f->deg;
   g_deg = g->deg;
   if (f_deg >= exp)
      f->deg = exp - 1;
   if (g_deg >= exp)
      g->deg = exp - 1;
   mpcx_mul (h, f, g);
   h_deg = h->deg;
   f->deg = f_deg;
   g->deg = g_deg;
   h->deg = h_deg;
      /* The previous line is needed if h is one of f or g. */
   if (h->deg >= exp) {
      h->deg = exp - 1;
      while (h->deg >= 0 && mpc_cmp_si (h->coeff [h->deg], 0) == 0)
         h->deg--;
   }
}

/*****************************************************************************/

void mpcx_sinv (mpcx_ptr h, mpcx_ptr f, const int exp) {
   /* computes h = f^{-1} mod X^exp by Newton iteration */

   int    overlap;
   mpcx_t h_local, tmp;
   int    exp_local;

   overlap = (h == f);
   if (overlap)
      mpcx_init2 (h_local, exp, h->prec);
   else
      mpcx_mv (h_local, h);
   mpcx_init2 (tmp, exp, h->prec);

   exp_local = 1;
   h_local->deg = 0;
   mpc_ui_div (h_local->coeff [0], 1, f->coeff [0], MPC_RNDNN);
   while (exp_local < exp) {
      exp_local = (2 * exp_local < exp ? 2 * exp_local : exp);
      mpcx_smul (tmp, h_local, f, exp_local);
      mpcx_si_sub (tmp, 2, tmp);
      mpcx_smul (h_local, h_local, tmp, exp_local);
   }

   if (overlap)
      mpcx_clear (h);
   mpcx_clear (tmp);
   mpcx_mv (h, h_local);
}
