/**************************************************************************/
/*                                                                        */
/* mpcx_sub                                                               */
/*                                                                        */
/* computes h = f - g                                                     */
/*                                                                        */
/* Copyright (C) 2007 Andreas Enge                                        */
/*                                                                        */
/* Last changes: September 17, 2007                                       */
/*                                                                        */
/**************************************************************************/

#include "mpcx-impl.h"

void mpcx_sub (mpcx_ptr h, mpcx_srcptr f, mpcx_srcptr g) {
   int i;

   h->deg = MAX (f->deg, g->deg);
   if (h->size < h->deg + 1)
      mpcx_realloc2 (h, h->deg + 1);

   for (i = f->deg; i > g->deg; i--)
      mpc_set (h->coeff [i], f->coeff [i], MPC_RNDNN);
   for (i = g->deg; i > f->deg; i--)
      mpc_neg (h->coeff [i], g->coeff [i], MPC_RNDNN);
   for (i = MIN (f->deg, g->deg); i >= 0; i--)
      mpc_sub (h->coeff [i], f->coeff [i], g->coeff [i], MPC_RNDNN);

   while (h->deg >= 0 && mpc_cmp_si (h->coeff [h->deg], 0) == 0)
      h->deg--;
}

/*****************************************************************************/

void mpcx_si_sub (mpcx_ptr h, const long int f, mpcx_srcptr g)

{
   int i;

   if (g->deg == -1) {
      if (f == 0)
         h->deg = -1;
      else {
         h->deg = 0;
         mpc_set_ui (h->coeff [0], f, MPC_RNDNN);
      }
   }
   else if (g->deg == 0 && mpc_cmp_si (g->coeff [0], f) == 0)
      h->deg = -1;
   else {
      if (h->size < g->deg + 1)
         mpcx_realloc2 (h, g->deg + 1);
      h->deg = g->deg;
      for (i = 0; i <= g->deg; i++)
         mpc_neg (h->coeff [i], g->coeff [i], MPC_RNDNN);
      mpc_add_ui (h->coeff [0], h->coeff [0], f, MPC_RNDNN);
   }
}
