/**************************************************************************/
/*                                                                        */
/* mpfrx_rev                                                               */
/*                                                                        */
/* computes h = X^exp * f (1/X); if exp is 0, then it is replaced by the  */
/* degree of f                                                            */
/*                                                                        */
/* Copyright (C) 2007 Andreas Enge                                        */
/*                                                                        */
/* Last changes: August 31, 2007                                          */
/*                                                                        */
/**************************************************************************/

#include "mpfrx-impl.h"

void mpfrx_rev (mpfrx_ptr h, mpfrx_srcptr f, const int exp) {
   int    overlap, exp_local, i;
   mpfrx_t h_local;

   if (exp == 0)
      exp_local = f->deg;
   else
      exp_local = exp;

   if (exp_local < f->deg) {
      printf ("*** Computing a reverse polynomial of too low ");
      printf ("order in 'mpfrx_rev'.\n");
      exit (1);
   }

   overlap = (h == f);
   if (overlap)
      mpfrx_init2 (h_local, exp_local + 1, h->prec);
   else {
      mpfrx_mv (h_local, h);
      if (h_local->size < exp_local + 1)
         mpfrx_realloc2 (h_local, exp_local + 1);
   }

   h_local->deg = exp_local;
   for (i = 0; i <= f->deg; i++)
      mpfr_set (h_local->coeff [exp_local - i], f->coeff [i], GMP_RNDN);
   for (i = exp_local - f->deg - 1; i >= 0; i--)
      mpfr_set_ui (h_local->coeff [i], 0, GMP_RNDN);

   while (h_local->deg >= 0
         && mpfr_cmp_si (h_local->coeff [h_local->deg], 0) == 0)
      h_local->deg--;
   if (overlap)
      mpfrx_clear (h);
   mpfrx_mv (h, h_local);
}
