/**************************************************************************/
/*                                                                        */
/* mpfrx_sub                                                               */
/*                                                                        */
/* computes h = f - g                                                     */
/*                                                                        */
/* Copyright (C) 2007 Andreas Enge                                        */
/*                                                                        */
/* Last changes: September 17, 2007                                       */
/*                                                                        */
/**************************************************************************/

#include "mpfrx-impl.h"

void mpfrx_sub (mpfrx_ptr h, mpfrx_srcptr f, mpfrx_srcptr g) {
   int i;

   h->deg = MAX (f->deg, g->deg);
   if (h->size < h->deg + 1)
      mpfrx_realloc2 (h, h->deg + 1);

   for (i = f->deg; i > g->deg; i--)
      mpfr_set (h->coeff [i], f->coeff [i], GMP_RNDN);
   for (i = g->deg; i > f->deg; i--)
      mpfr_neg (h->coeff [i], g->coeff [i], GMP_RNDN);
   for (i = MIN (f->deg, g->deg); i >= 0; i--)
      mpfr_sub (h->coeff [i], f->coeff [i], g->coeff [i], GMP_RNDN);

   while (h->deg >= 0 && mpfr_cmp_si (h->coeff [h->deg], 0) == 0)
      h->deg--;
}

/*****************************************************************************/

void mpfrx_si_sub (mpfrx_ptr h, const long int f, mpfrx_srcptr g)

{
   int i;

   if (g->deg == -1) {
      if (f == 0)
         h->deg = -1;
      else {
         h->deg = 0;
         mpfr_set_ui (h->coeff [0], f, GMP_RNDN);
      }
   }
   else if (g->deg == 0 && mpfr_cmp_si (g->coeff [0], f) == 0)
      h->deg = -1;
   else {
      if (h->size < g->deg + 1)
         mpfrx_realloc2 (h, g->deg + 1);
      h->deg = g->deg;
      for (i = 0; i <= g->deg; i++)
         mpfr_neg (h->coeff [i], g->coeff [i], GMP_RNDN);
      mpfr_add_ui (h->coeff [0], h->coeff [0], f, GMP_RNDN);
   }
}
