/**************************************************************************/
/*                                                                        */
/* include file for polynomials over mpcx                                 */
/*                                                                        */
/* Copyright (C) 2007 Andreas Enge                                        */
/*                                                                        */
/* Last changes: September 17, 2007                                       */
/*                                                                        */
/**************************************************************************/

#ifndef __MPCX_H
#define __MPCX_H
#include <stdio.h>
#include <stdlib.h>
#include "gmp.h"
#include "mpc.h"

typedef struct {
   int    size;
   /* the number of currently allocated coefficients. High level          */
   /* arithmetic functions may increase this number if more space is      */
   /* needed. Coefficients are only freed when the whole polynomial is.   */
   /* Allocated coefficients are immediately initialised with mpc_init.   */
   int   deg;
   /* A degree of -1 indicates the zero polynomial.                       */
   mp_prec_t prec;
   /* the precision used for initialising all the coefficients            */
   mpc_t *coeff;
}
__mpcx_struct;

typedef __mpcx_struct mpcx_t[1];
typedef __mpcx_struct *mpcx_ptr;
typedef __gmp_const __mpcx_struct *mpcx_srcptr;


#if defined (__cplusplus)
extern "C" {
#endif
extern void mpcx_init2 (mpcx_ptr f, const int size, const mp_prec_t prec);
extern void mpcx_set (mpcx_ptr h, mpcx_srcptr f);
extern void mpcx_init_set (mpcx_ptr h, mpcx_srcptr f);
extern void mpcx_clear (mpcx_ptr f);

extern void mpcx_add (mpcx_ptr h, mpcx_srcptr f, mpcx_srcptr g);
extern void mpcx_sub (mpcx_ptr h, mpcx_srcptr f, mpcx_srcptr g);
extern void mpcx_si_sub (mpcx_ptr h, const long int c, mpcx_srcptr f);
extern void mpcx_neg (mpcx_ptr h, mpcx_srcptr f);

extern void mpcx_mul (mpcx_ptr h, mpcx_srcptr f, mpcx_srcptr g);

extern void mpcx_rem (mpcx_ptr r, mpcx_srcptr f, mpcx_srcptr g);

extern void mpcx_reconstruct (mpcx_t rop, mpcx_t *factors, int no);
extern void mpcx_multieval (mpc_t *values, mpc_t *args, int no,
                            mpcx_srcptr f);
extern void mpcx_root (mpc_t root, mpcx_t f);

extern void mpcx_print (mpcx_srcptr h);
#if defined (__cplusplus)
}
#endif
#endif /* ifndef __MPCX_H */
