/**************************************************************************/
/*                                                                        */
/* include file for polynomials over mpfrx                                 */
/*                                                                        */
/* Copyright (C) 2007 Andreas Enge                                        */
/*                                                                        */
/* Last changes: September 17, 2007                                       */
/*                                                                        */
/**************************************************************************/

#ifndef __MPFRX_H
#define __MPFRX_H
#include <stdio.h>
#include <stdlib.h>
#include "gmp.h"
#include "mpfr.h"

typedef struct {
   int    size;
   /* the number of currently allocated coefficients. High level          */
   /* arithmetic functions may increase this number if more space is      */
   /* needed. Coefficients are only freed when the whole polynomial is.   */
   /* Allocated coefficients are immediately initialised with mpfr_init.   */
   int   deg;
   /* A degree of -1 indicates the zero polynomial.                       */
   mp_prec_t prec;
   /* the precision used for initialising all the coefficients            */
   mpfr_t *coeff;
}
__mpfrx_struct;

typedef __mpfrx_struct mpfrx_t[1];
typedef __mpfrx_struct *mpfrx_ptr;
typedef __gmp_const __mpfrx_struct *mpfrx_srcptr;


#if defined (__cplusplus)
extern "C" {
#endif
extern void mpfrx_init2 (mpfrx_ptr f, const int size, const mp_prec_t prec);
extern void mpfrx_set (mpfrx_ptr h, mpfrx_srcptr f);
extern void mpfrx_init_set (mpfrx_ptr h, mpfrx_srcptr f);
extern void mpfrx_clear (mpfrx_ptr f);

extern void mpfrx_add (mpfrx_ptr h, mpfrx_srcptr f, mpfrx_srcptr g);
extern void mpfrx_sub (mpfrx_ptr h, mpfrx_srcptr f, mpfrx_srcptr g);
extern void mpfrx_si_sub (mpfrx_ptr h, const long int c, mpfrx_srcptr f);
extern void mpfrx_neg (mpfrx_ptr h, mpfrx_srcptr f);

extern void mpfrx_mul (mpfrx_ptr h, mpfrx_srcptr f, mpfrx_srcptr g);

extern void mpfrx_rem (mpfrx_ptr r, mpfrx_srcptr f, mpfrx_srcptr g);

extern void mpfrx_reconstruct (mpfrx_t rop, mpfrx_t *factors, int no);
extern void mpfrx_multieval (mpfr_t *values, mpfr_t *args, int no,
                            mpfrx_srcptr f);
extern void mpfrx_root (mpfr_t root, mpfrx_t f);

extern void mpfrx_print (mpfrx_srcptr h);
#if defined (__cplusplus)
}
#endif
#endif /* ifndef __MPFRX_H */
